@@ -20,13 +20,18 @@ from functools import partial
 import multiprocessing
 import pandas as pd
 import pydot
+import igraph as ig
+import matplotlib.cm as cm
+import matplotlib.colors as mcolors
+from matplotlib.patches import Wedge
+from matplotlib.patches import Arc
 
 __all__ = ['search', 'bnet', 'cext', 'dataset', 'bdm', 'bnetload', 'check_type', 'cond_normal', 'conditional_sampl', 'continuous_sampler','conv','conv_col','conv_row','cpt',
            'downstream_sampler', 'downstream_sampler1', 'factor_indx', 'factor_str', 'joint_prob', 'loader', 'make_A', 'mdl', 'mu', 'random_adjmat', 'random_dist',
            'test','upstream_sampler', 'read_input_file','getContact2discret', 'runParallel', 'datawrite', 'read_tsv', 'Residue', 
            'transformRes', 'get_unique_pair', 'get_trj_s', 'get_traj_p', 'remove_Neighbors',  'getGraphProp', 'load_graph',
            'is_valid_node', 'create_adjacency_matrix', 'calculate_hamming_distance_between_graphs', 'calculate_hamming_distances_per_node',
-           'convert_bn_to_igraph']
+           'convert_bn_to_igraph', 'plot_hamming_distances']
 
 def bnetload(structure):
     """ structure can be a csv file or and adjacency matrix """
@@ -542,7 +547,7 @@ class Residue():
                 'PHE': 'F', 'ASN': 'N', 'GLY': 'G', 'HIS': 'H', 'HSD': 'H',
                 'HID': 'H', 'HIE': 'H', 'HIP': 'H', 'LEU': 'L', 'ARG': 'R', 
                 'TRP': 'W', 'ALA': 'A', 'VAL': 'V', 'GLU': 'E', 'GLH': 'E', 
-                'TYR': 'Y', 'MET': 'M', 'ZD7': 'Z', 'NMA': 'X' }
+                'TYR': 'Y', 'MET': 'M', 'ZD7': 'Z', 'NMA': 'X', 'LIG': 'LIG' }
 
     def shRes(self,x):
         #print(self.d.keys())
@@ -588,7 +593,7 @@ def remove_Neighbors(contacts,N):
 
 #Main Function below
 
-def getContact2discret(tsvfile='input.tsv',neighbors=1,csv_out='trajectory.csv',nproc=28):
+def getContact2discret(tsvfile='input.tsv',neighbors=1,csv_out='pairwise_residue_contacts.csv',nproc=28):
     
     # Bulk of this function is this step which is reading tsv and converting to contact matrix
     h,tsv_data=read_tsv(tsvfile)
@@ -612,11 +617,11 @@ def getContact2discret(tsvfile='input.tsv',neighbors=1,csv_out='trajectory.csv',
 def read_input_file(f):
     if f[-3:]=='tsv':
         discreteData=getContact2discret(f)
-        dt=bnomics.dutils.loader(discreteData)
+        dt=loader(discreteData)
     else:
-        dt=bnomics.dutils.loader(filename)
+        dt=loader(f)
         if np.any(np.array([np.unique(x).size/len(x) for  x in dt.data.T])>0.95):
-            dt.quantize_all()
+            dt.quantize_all(bins=8)
     return dt
 
 def loader(data,sep=',',skip_header=0, rowskip=[], colskip=[], axis=1, names=1, fromstring=0):
@@ -743,7 +748,7 @@ class dataset:
         self.data=data
         self.arity=arity
 
-    def bin_quantize(self, variables=[], bins=3, min_const_samples_bin_size=1.0/3):
+    def bin_quantize(self, variables=[], bins=3):
         """ Attempt max entropy binning quantization 
 
             variables:  a list or an array as in variables=[1,3,5]
@@ -751,6 +756,7 @@ class dataset:
             min_const_samples_bin_size: determines the min count of
             constant samples that should be given a category of its own
         """
+        min_const_samples_bin_size=1.0/bins
         self.edges=np.zeros((self.arity.size,bins+1))
         for i in variables:
             un_cnt=np.unique(self.data[:,i],return_counts=True)
@@ -805,7 +811,7 @@ class dataset:
                 self.data[inds,i]=un.index(j)
                 
 
-    def quantize_all(self, cond = 5, bins=3):
+    def quantize_all(self, cond = 5, bins=8):
         """ Discretize everything with arity>5 without thinking. 
         """
         self.requantize(np.where(self.arity<=cond)[0])
@@ -1970,7 +1976,7 @@ def create_adjacency_matrix(graph, nodes):
     return adjacency_matrix
 
 def calculate_hamming_distance_between_graphs(dot_file_path_1, dot_file_path_2):
-    """Calculate the Hamming distance between two matrices for each node."""
+    """Calculate the Hamming distance between two matrices."""
     graph1=load_graph(dot_file_path_1)
     graph2=load_graph(dot_file_path_2)
     nodes = {node.get_name().strip().strip('"') for node in graph1.get_nodes() if is_valid_node(node.get_name().strip().strip('"'))}
@@ -1984,8 +1990,8 @@ def calculate_hamming_distance_between_graphs(dot_file_path_1, dot_file_path_2):
         node_index = node_to_index[node]
         distance = np.sum(np.abs(matrix1[node_index, :] - matrix2[node_index, :]))
         hamming_distances += distance
-
-    return hamming_distances
+    print(f'Weighted Hamming distance between two graphs: {hamming_distances}')
+    #return hamming_distances
     
 def calculate_hamming_distances_per_node(dot_file_path_1, dot_file_path_2, output_filename):
     """Calculate the Hamming distance for each node between two matrices."""
@@ -2004,39 +2010,52 @@ def calculate_hamming_distances_per_node(dot_file_path_1, dot_file_path_2, outpu
         distance = np.sum(np.abs(matrix1[node_index, :] - matrix2[node_index, :]))
         hamming_distances[node] = distance
         
-    df = pd.DataFrame(list(hamming_distances.items()), columns=['Node', 'Hamming Distance'])
+    df = pd.DataFrame(list(hamming_distances.items()), columns=['Node', 'Weighted Hamming Distance'])
     df.to_csv(output_filename, index=False)
     
     return hamming_distances
 
 def convert_bn_to_igraph(srch,tol=0,directed=True,fout=False,format="pickle"):
-
     g = ig.Graph(directed=directed)
-
     bn=srch.BN
-
     g.add_vertices(bn.node_index.size)
-
     g.vs["label"]=bn.node_names
-
     ew=[]
-
     for child in bn.node_index:
-
         for parent in bn.pnodes[child]:
-
             edge_score=srch.score_edge(child,parent)
-
             if edge_score>tol:
-
                 g.add_edge(parent,child)
-
                 ew.append(edge_score)
-
     g.es["weight"]=ew
-
     if fout:
-
         g.save(fout,format=format)
-
     return g
+
+def plot_hamming_distances(hamming_distances, value_filter=0, figsize=(25,25), fout=False):
+    """Plot hamming distances in a circular graph representation, only for values greater than 0."""
+    filtered_distances = {key: value for key, value in hamming_distances.items() if value > value_filter}
+    sorted_data = sorted(filtered_distances.items(), key=lambda item: item[1])
+    offset = np.pi / 2
+    starting_radius = 0.5
+    segment_width = 1
+    arc_spacing = 0.1
+    text_offset = 0.1
+    colors = plt.cm.viridis(np.linspace(0, 1, len(filtered_distances)))
+    fig, ax = plt.subplots(figsize=figsize, subplot_kw=dict(polar=True))
+    
+    for i, (key, num) in enumerate(sorted_data):
+        end_angle = (num / sum(hamming_distances.values())) * 2 * np.pi * 5
+        end_angle = min(end_angle, 2 * np.pi)
+        t = np.linspace(offset, end_angle + offset, 100)
+        r = np.full_like(t, starting_radius + i * (segment_width + arc_spacing))
+        ax.plot(t, r, lw=segment_width * 11, color='black', solid_capstyle='butt')
+        ax.plot(t, r, lw=segment_width * 10, color=colors[i], solid_capstyle='butt')
+        ax.text(offset, r[0], key, ha='left', va='center', fontsize=9, color='black')
+    ax.set_frame_on(False)
+    ax.axes.get_yaxis().set_visible(False)
+    ax.axes.get_xaxis().set_visible(False)
+    
+    if fout:
+        plt.savefig(fout, dpi=500)
+    plt.show()
